@@ -263,9 +263,9 @@ uerr_t ftp_get_file_chunk(connection_data * connection)
 	    char dhost[256];
 	    unsigned short dport;
 
-	    sprintf(dhost, "%d.%d.%d.%d",
-		    pasv_addr[0], pasv_addr[1], pasv_addr[2],
-		    pasv_addr[3]);
+	    snprintf(dhost, sizeof(dhost), "%d.%d.%d.%d",
+		     pasv_addr[0], pasv_addr[1], pasv_addr[2],
+		     pasv_addr[3]);
 	    dport = (pasv_addr[4] << 8) + pasv_addr[5];
 	    /*
 	     * message("Connecting to %s %d",dhost,dport);          
@@ -564,9 +564,10 @@ uerr_t ftp_get_file_to_end(connection_data * connection)
 	{
 	    char dhost[256];
 	    unsigned short dport;
-	    sprintf(dhost, "%d.%d.%d.%d",
-		    pasv_addr[0], pasv_addr[1], pasv_addr[2],
-		    pasv_addr[3]);
+
+	    snprintf(dhost, sizeof(dhost), "%d.%d.%d.%d",
+		     pasv_addr[0], pasv_addr[1], pasv_addr[2],
+		     pasv_addr[3]);
 	    dport = (pasv_addr[4] << 8) + pasv_addr[5];
 
 	    err =
@@ -873,8 +874,9 @@ void ftp_loop(connection_data * connection)
 
 uerr_t ftp_retr_fsize_known(connection_data * connection, FILE * fp)
 {
-    long total = 0, bytes_read = 0;
-    long temp = connection->remote_endpos - connection->remote_startpos;
+    int bytes_read = 0;
+    off_t total = 0;
+    off_t temp = connection->remote_endpos - connection->remote_startpos;
 
     while (temp > 0)
     {
@@ -976,7 +978,7 @@ uerr_t ftp_retr_fsize_known(connection_data * connection, FILE * fp)
     connection->status = COMPLETED;
 
     message("download for this connection completed");
-    message("%s : %ld received", connection->localfile, total);
+    message("%s : %Ld received", connection->localfile, (long long) total);
     return FTPOK;
 }
 
@@ -984,7 +986,7 @@ uerr_t ftp_retr_fsize_known(connection_data * connection, FILE * fp)
 
 uerr_t ftp_retr_fsize_notknown(connection_data * connection, FILE * fp)
 {
-    long bytes_read;
+    off_t bytes_read;
     do
     {
 	bytes_read =
@@ -1025,8 +1027,8 @@ uerr_t ftp_retr_fsize_notknown(connection_data * connection, FILE * fp)
 
     connection->status = COMPLETED;
     message("download for this connection completed");
-    message("%s : %ld received", connection->localfile,
-	    connection->remote_bytes_received);
+    message("%s : %Ld received", connection->localfile,
+	    (long long) connection->remote_bytes_received);
     return FTPOK;
 }
 
