@@ -137,7 +137,7 @@ uerr_t ftp_get_reply(int sock, char *szBuffer)
 {
     int cont = 0;
     int code;
-    char * tmp = (char *)alloca(FTP_BUFFER_SIZE);
+    char * tmp = alloca(FTP_BUFFER_SIZE);
 
     memset(szBuffer, 0, FTP_BUFFER_SIZE);
 
@@ -189,7 +189,7 @@ uerr_t ftp_ascii(int sock)
     char szBuffer[FTP_BUFFER_SIZE];
     uerr_t err;
 
-    sprintf(szBuffer, "TYPE A\r\n");
+    snprintf(szBuffer, sizeof(szBuffer), "TYPE A\r\n");
 
     err = ftp_send_msg(sock, szBuffer, strlen(szBuffer));
     if (err != FTPOK)
@@ -217,7 +217,7 @@ uerr_t ftp_binary(int sock)
     char szBuffer[FTP_BUFFER_SIZE];
     uerr_t err;
 
-    sprintf(szBuffer, "TYPE I\r\n");
+    snprintf(szBuffer, sizeof(szBuffer), "TYPE I\r\n");
     err = ftp_send_msg(sock, szBuffer, strlen(szBuffer));
     if (err != FTPOK)
     {
@@ -273,7 +273,12 @@ uerr_t ftp_list(int sock, char *file)
     char szBuffer[FTP_BUFFER_SIZE];
     uerr_t err;
 
-    sprintf(szBuffer, "LIST %s\r\n", file);
+    if (snprintf(szBuffer, sizeof(szBuffer),
+		 "LIST %s\r\n",
+		 file) >= sizeof(szBuffer))
+    {
+        return NOTENOUGHMEM; /* FIXME: Better error */
+    }
     err = ftp_send_msg(sock, szBuffer, strlen(szBuffer));
     if (err != FTPOK)
     {
@@ -304,7 +309,12 @@ uerr_t ftp_retr(int sock, char *file)
     char szBuffer[FTP_BUFFER_SIZE];
     uerr_t err;
 
-    sprintf(szBuffer, "RETR %s\r\n", file);
+    if (snprintf(szBuffer, sizeof(szBuffer),
+		 "RETR %s\r\n",
+		 file) >= sizeof(szBuffer))
+    {
+        return NOTENOUGHMEM; /* FIXME: Better error */
+    }
     err = ftp_send_msg(sock, szBuffer, strlen(szBuffer));
     if (err != FTPOK)
     {
@@ -342,7 +352,7 @@ uerr_t ftp_pasv(int sock, unsigned char *addr)
     unsigned char *p;
     int i;
 
-    sprintf(szBuffer, "PASV\r\n");
+    snprintf(szBuffer, sizeof(szBuffer), "%s", "PASV\r\n");
     err = ftp_send_msg(sock, szBuffer, strlen(szBuffer));
     if (err != FTPOK)
     {
@@ -384,12 +394,17 @@ uerr_t ftp_pasv(int sock, unsigned char *addr)
     return FTPOK;
 }
 
-uerr_t ftp_rest(int sock, long bytes)
+uerr_t ftp_rest(int sock, off_t bytes)
 {
     char szBuffer[FTP_BUFFER_SIZE];
     uerr_t err;
 
-    sprintf(szBuffer, "REST %ld\r\n", bytes);
+    if (snprintf(szBuffer, sizeof(szBuffer),
+		 "REST %Ld\r\n", (long long)
+		 bytes) >= sizeof(szBuffer))
+    {
+        return NOTENOUGHMEM; /* FIXME: Better error */
+    }
     err = ftp_send_msg(sock, szBuffer, strlen(szBuffer));
     if (err != FTPOK)
     {
@@ -417,7 +432,12 @@ uerr_t ftp_cwd(int sock, char *dir)
     char szBuffer[FTP_BUFFER_SIZE];
     uerr_t err;
 
-    sprintf(szBuffer, "CWD %s\r\n", dir);
+    if (snprintf(szBuffer, sizeof(szBuffer),
+		 "CWD %s\r\n",
+		 dir) >= sizeof(szBuffer))
+    {
+        return NOTENOUGHMEM; /* FIXME: Better error */
+    }
     err = ftp_send_msg(sock, szBuffer, strlen(szBuffer));
     if (err != FTPOK)
     {
@@ -432,7 +452,7 @@ uerr_t ftp_cwd(int sock, char *dir)
 
 
     /*
-     * check and see wether the CWD succeeded 
+     * check and see whether the CWD succeeded 
      */
     if (*szBuffer == '5')
     {
@@ -460,7 +480,7 @@ uerr_t ftp_pwd(int sock, char *dir)
     uerr_t err;
     char *r, *l;
 
-    sprintf(szBuffer, "PWD");
+    snprintf(szBuffer, sizeof(szBuffer), "PWD\r\n");
     err = ftp_send_msg(sock, szBuffer, strlen(szBuffer));
     if (err != FTPOK)
     {
@@ -606,9 +626,9 @@ uerr_t ftp_get_listen_socket(int control_sock, int *listen_sock)
 
 #define  UC(b)  (((int)b)&0xff)
 
-    sprintf(command, "PORT %d,%d,%d,%d,%d,%d\r\n",
-	    UC(ipaddr[0]), UC(ipaddr[1]), UC(ipaddr[2]), UC(ipaddr[3]),
-	    UC(port[0]), UC(port[1]));
+    snprintf(command, sizeof(command), "PORT %d,%d,%d,%d,%d,%d\r\n",
+	     UC(ipaddr[0]), UC(ipaddr[1]), UC(ipaddr[2]), UC(ipaddr[3]),
+	     UC(port[0]), UC(port[1]));
 
     /*
      * handle error 
@@ -630,7 +650,12 @@ uerr_t ftp_login(int control_sock, const char *username,
 {
     uerr_t err;
     char szBuffer[FTP_BUFFER_SIZE];
-    sprintf(szBuffer, "USER %s\r\n", username);
+    if (snprintf(szBuffer, sizeof(szBuffer),
+		 "USER %s\r\n",
+		 username) >= sizeof(szBuffer))
+    {
+        return NOTENOUGHMEM; /* FIXME: Better error */
+    }
 
     err = ftp_send_msg(control_sock, szBuffer, strlen(szBuffer));
     if (err != FTPOK)
@@ -663,7 +688,12 @@ uerr_t ftp_login(int control_sock, const char *username,
      * send password 
      */
 
-    sprintf(szBuffer, "PASS %s\r\n", passwd);
+    if (snprintf(szBuffer, sizeof(szBuffer),
+		 "PASS %s\r\n",
+		 passwd) >= sizeof(szBuffer))
+    {
+        return NOTENOUGHMEM; /* FIXME: Better error */
+    }
     err = ftp_send_msg(control_sock, szBuffer, strlen(szBuffer));
 
     if (err != FTPOK)
@@ -744,7 +774,12 @@ uerr_t ftp_get_file_info(int control_sock, char *fname, ftp_stat_t * fs)
      * SIZE command and if the server doesn't support it * we fall back on
      * the LIST command..... 
      */
-    sprintf(szBuffer, "SIZE %s\r\n", fname);
+    if (snprintf(szBuffer, sizeof(szBuffer),
+		 "SIZE %s\r\n",
+		 fname) >= sizeof(szBuffer))
+    {
+        return NOTENOUGHMEM; /* FIXME: Better error */
+    }
     err = ftp_send_msg(control_sock, szBuffer, strlen(szBuffer));
     if (err != FTPOK)
     {
@@ -764,7 +799,17 @@ uerr_t ftp_get_file_info(int control_sock, char *fname, ftp_stat_t * fs)
 				 * everything ok... 
 				 */
     {
-	sscanf(szBuffer + 3, "%ld", &(fs->fp.size));
+#if SIZEOF_OFF_T == 4
+        sscanf(szBuffer + 3, "%ld", &(fs->fp.size));
+#elif SIZEOF_OFF_T == 8
+	sscanf(szBuffer + 3, "%Ld", &(fs->fp.size));
+#else
+	{
+	    err = NOTENOUGHMEM;
+	    return err;
+	}
+#endif
+
 	fs->fp.flagtryretr = 1;
 	return FTPOK;
     } else if (*szBuffer == '5')  /** A error hapenned */
@@ -804,9 +849,10 @@ uerr_t ftp_get_file_info(int control_sock, char *fname, ftp_stat_t * fs)
 	{
 	    char dhost[256];
 	    unsigned short dport;
-	    sprintf(dhost, "%d.%d.%d.%d",
-		    pasv_addr[0], pasv_addr[1], pasv_addr[2],
-		    pasv_addr[3]);
+
+	    snprintf(dhost, sizeof(dhost), "%d.%d.%d.%d",
+		     pasv_addr[0], pasv_addr[1], pasv_addr[2],
+		     pasv_addr[3]);
 	    dport = (pasv_addr[4] << 8) + pasv_addr[5];
 
 	    err = connect_to_server(&data_sock, dhost, dport, rt.timeout);
