@@ -103,11 +103,10 @@ char *get_string_ahref(char *buf, char *out)
 
 char *prepare_lycos_url(urlinfo * url_data, char * ftps_loc)
 {
+#define LYCOS_URL "?form=advanced&query=%s&doit=Search&type=Exact+search&hits=%d&matches=&hitsprmatch=&limdom=&limpath=&limsize1=%Ld&limsize2=%Ld&f1=Host&f2=Path&f3=Size&f4=-&f5=-&f6=-&header=none&sort=none&trlen=20"
 
     char *lycos_url_buf;
-    int lycos_url_len = strlen(ftps_loc) +
-	strlen
-	("?form=advanced&query=%s&doit=Search&type=Exact+search&hits=%d&matches=&hitsprmatch=&limdom=&limpath=&limsize1=%d&limsize2=%d&limtime1=&limtime2=&f1=Host&f2=Path&f3=Size&f4=-&f5=-&f6=-&header=none&sort=none&trlen=20");
+    int lycos_url_len;
 
     /*    int len,len1;
        char *p, *p1,*lycos_dir;
@@ -116,13 +115,24 @@ char *prepare_lycos_url(urlinfo * url_data, char * ftps_loc)
 
 
     /* Okay lets now construct the URL we want to do lycos */
-    lycos_url_buf =
-	(char *) kmalloc(lycos_url_len + strlen(url_data->file) + 300);
+    lycos_url_len = snprintf(NULL, 0,
+			     "%s" LYCOS_URL,
+			     ftps_loc, url_data->file, rt.ftps_mirror_req_n,
+			    (long long) url_data->file_size, 
+			    (long long) url_data->file_size);
+    if (lycos_url_len <= 0)
+      die("Unable to calculate buffer length for Lycos search\n");
 
-    sprintf(lycos_url_buf,
-	    "%s?form=advanced&query=%s&doit=Search&type=Exact+search&hits=%d&matches=&hitsprmatch=&limdom=&limpath=&limsize1=%ld&limsize2=%ld&f1=Host&f2=Path&f3=Size&f4=-&f5=-&f6=-&header=none&sort=none&trlen=20",
-	    ftps_loc, url_data->file, rt.ftps_mirror_req_n,
-	    url_data->file_size, url_data->file_size);
+    lycos_url_len++; /* nul */
+    lycos_url_buf = kmalloc(lycos_url_len);
+
+    snprintf(lycos_url_buf, lycos_url_len,
+	     "%s" LYCOS_URL,
+	     ftps_loc, url_data->file, rt.ftps_mirror_req_n,
+	     (long long) url_data->file_size, 
+	     (long long) url_data->file_size);
+
+#undef LYCOS_URL
 
 /* if we have a dir
    if(strcmp(url_data->dir,"pub")==0)
@@ -259,8 +269,7 @@ uerr_t parse_html_mirror_list(urlinfo * url_data,char *p,struct ftp_mirror ** pm
 
     /* Allocate +1 because we need to add the user specified server as well
      */
-    ftp_mirrors =
-	(ftp_mirror *) kmalloc(sizeof(ftp_mirror) * ((*num_servers) + 1));
+    ftp_mirrors = kmalloc(sizeof(ftp_mirror) * ((*num_servers) + 1));
     assert(ftp_mirrors != 0);
 
     for (k = 0; k < *num_servers; k++)
@@ -296,11 +305,11 @@ uerr_t parse_html_mirror_list(urlinfo * url_data,char *p,struct ftp_mirror ** pm
     {
 
 	ftp_mirrors[k].full_name =
-	    (char *) kmalloc(strlen(ftp_mirrors[k].server_name) +
-			     strlen(ftp_mirrors[k].path) +
-			     strlen(ftp_mirrors[k].file_name) + 13);
-	sprintf(ftp_mirrors[k].full_name, "%s%s:21/%s%s%s", "ftp://",
-		ftp_mirrors[k].server_name, ftp_mirrors[k].path, "/",
+	    kmalloc(strlen(ftp_mirrors[k].server_name) +
+		    strlen(ftp_mirrors[k].path) +
+		    strlen(ftp_mirrors[k].file_name) + 12);
+	sprintf(ftp_mirrors[k].full_name, "ftp://%s:21/%s/%s",
+		ftp_mirrors[k].server_name, ftp_mirrors[k].path,
 		ftp_mirrors[k].file_name);
 
 	debug_prz("%s\n", ftp_mirrors[k].full_name);
@@ -680,6 +689,7 @@ uerr_t get_mirror_info(urlinfo * u, http_stat_t * hs, char **ret_buf)
 
     int sock, i;
     char buffer[HTTP_BUFFER_SIZE];
+    char *req = NULL;
     char *user, *passwd, *wwwauth;
     char *p, *p1, *p2;
     uerr_t err;
@@ -709,7 +719,7 @@ uerr_t get_mirror_info(urlinfo * u, http_stat_t * hs, char **ret_buf)
 		    parseurl(constructed_newloc, u, 0);
 		  if (err != URLOK)
 		    {
-		      die("The server returned location is syntatically wrong: %s!", constructed_newloc);
+		      die("The server returned location is syntactically wrong: %s!", constructed_newloc);
 		    }
 
 
@@ -747,7 +757,7 @@ uerr_t get_mirror_info(urlinfo * u, http_stat_t * hs, char **ret_buf)
 	if (hs->statcode == HTTP_NOT_FOUND)
 	{
 	    message
-		("Error: Uanable to get mirror info from the server: Not found!\n");
+		("Error: Unable to get mirror info from the server: Not found!\n");
 	} else
 	{
 	    message
@@ -795,19 +805,41 @@ uerr_t get_mirror_info(urlinfo * u, http_stat_t * hs, char **ret_buf)
     } else
 	wwwauth = 0;
 
-    /* Handle referer */
+    /* TODO: Handle referer? */
 
     /* We will get http info about the file by calling http_fetch_headers 
-       with HEAD */
+       with GET */
 
+   {
+       const char reqfmt[] =
+	 "GET %s HTTP/1.0\r\n"
+	 "User-Agent: %s\r\n"
+	 "Host: %s\r\n"
+	 "Accept: */*\r\n"
+	 "%s\r\n";
+       int reqlen;
 
-    sprintf(buffer,
-	    "GET %s HTTP/1.0\r\nUser-Agent: %s%s\r\nHost: %s\r\nAccept: */*\r\n%s\r\n",
-	    u->path, PACKAGE_NAME, VERSION, u->host,
-	    wwwauth ? wwwauth : "");
+       reqlen = snprintf(NULL, 0,
+			 reqfmt,
+			 u->path, USER_AGENT, u->host, 
+			 wwwauth ? wwwauth : "");
+       if (reqlen <= 0)
+	 die("Unable to calculate buffer length for HTTP GET request\n");
 
-    err = http_fetch_headers(sock, u, hs, buffer);
+       reqlen++; /* nul */
+       req = kmalloc(reqlen);
 
+       snprintf(req, reqlen,
+		reqfmt,
+		u->path, USER_AGENT, u->host, 
+		wwwauth ? wwwauth : "");
+   }
+
+    debug_prz("HTTP request= %s\n", req);
+
+    err = http_fetch_headers(sock, u, hs, req);
+
+    kfree(req);
     if (wwwauth)
 	free(wwwauth);
 
@@ -824,13 +856,13 @@ uerr_t get_mirror_info(urlinfo * u, http_stat_t * hs, char **ret_buf)
     }
 
     /* Ok start fetching the data */
-    p1 = p = (char *) kmalloc(HTTP_BUFFER_SIZE + 1);
+    p1 = p = kmalloc(HTTP_BUFFER_SIZE + 1);
     p_len = HTTP_BUFFER_SIZE + 1;
     total = 0;
     do
     {
 
-	ret = krecv(sock, buffer, HTTP_BUFFER_SIZE, 0, rt.timeout);
+	ret = krecv(sock, buffer, sizeof(buffer), 0, rt.timeout);
 	if (ret > 0)
 	{
 	    p2 = grow_buffer(p, p1, &p_len, ret);
