@@ -78,8 +78,7 @@ void clean_httpsock(void *cdata)
 
 uerr_t http_get_file_chunk(connection_data * connection)
 {
-
-    char buffer[HTTP_BUFFER_SIZE];
+    char *req = NULL;
     char *user, *passwd, *wwwauth, *referer=0;
     int tos = IPTOS_THROUGHPUT;
     netrc_entry *netrc_ent;
@@ -172,9 +171,10 @@ uerr_t http_get_file_chunk(connection_data * connection)
 
     if (strlen(user) || strlen(passwd))
     {
-	/*Construct the necessary header */
+	/* Construct the necessary header */
 	wwwauth = get_basic_auth_str(user, passwd);
-	message("Authenticating as user %s password %s", user, passwd);
+	message("Authenticating as user %s (%s password)", user,
+		(strlen(passwd))?"using":"no");
 
 	debug_prz("Authentification string=%s\n", wwwauth);
     } else
@@ -182,27 +182,55 @@ uerr_t http_get_file_chunk(connection_data * connection)
 
     if(connection->u.referer)
       {
-	referer= (char *) alloca(13+strlen(connection->u.referer));
+	referer= alloca(13+strlen(connection->u.referer));
 	sprintf(referer, "Referer: %s\r\n", connection->u.referer);
       }
 
     /*
      * get the headers by sending GET 
      */
-    sprintf(buffer,
-	    "GET %s HTTP/1.0\r\nUser-Agent: %s%s\r\nHost: %s\r\nAccept: */*\r\nRange: bytes=%ld-%ld\r\n%s%s\r\n",
-	    connection->u.path, PACKAGE_NAME, VERSION,
-	    connection->u.host,
-	    connection->remote_startpos, connection->remote_endpos,
-	    referer ? referer : "",
-	    wwwauth ? wwwauth : "");
 
+    {
+      const char reqfmt[] =
+	"GET %s HTTP/1.0\r\n"
+	"User-Agent: %s\r\n"
+	"Host: %s\r\n"
+	"Accept: */*\r\n"
+	"Range: bytes=%Ld-%Ld\r\n"
+	"%s%s\r\n";
+      int reqlen;
 
-    debug_prz("HTTP request= %s\n", buffer);
+      reqlen = snprintf(NULL, 0,
+			reqfmt,
+			connection->u.path, USER_AGENT,
+			connection->u.host,
+			(long long) connection->remote_startpos,
+			(long long) connection->remote_endpos,
+			referer ? referer : "",
+			wwwauth ? wwwauth : "");
+
+      if (reqlen <= 0)
+	die("Unable to calculate buffer length for HTTP GET request\n");
+
+      reqlen++; /* nul */
+      req = kmalloc(reqlen);
+
+      snprintf(req, reqlen,
+	       reqfmt,
+	       connection->u.path, USER_AGENT,
+	       connection->u.host,
+	       connection->remote_startpos, connection->remote_endpos,
+	       referer ? referer : "",
+	       wwwauth ? wwwauth : "");
+    }
+
+    debug_prz("HTTP request= %s\n", req);
 
     err =
 	http_fetch_headers(connection->http_sock, &(connection->u),
-			   &(connection->hs), buffer);
+			   &(connection->hs), req);
+
+    kfree(req);
     if (wwwauth)
 	free(wwwauth);
 
@@ -265,7 +293,7 @@ uerr_t http_get_file_chunk(connection_data * connection)
 
 uerr_t http_get_complete_file(connection_data * connection)
 {
-    char buffer[HTTP_BUFFER_SIZE];
+    char *req = NULL;
     char *user, *passwd, *wwwauth, *referer=0;
     int tos = IPTOS_THROUGHPUT;
     netrc_entry *netrc_ent;
@@ -344,7 +372,7 @@ uerr_t http_get_complete_file(connection_data * connection)
 
    if(connection->u.referer)
       {
-	referer= (char *) alloca(13+strlen(connection->u.referer));
+	referer= alloca(13+strlen(connection->u.referer));
 	sprintf(referer, "Referer: %s\r\n", connection->u.referer);
       }
 
@@ -357,17 +385,41 @@ uerr_t http_get_complete_file(connection_data * connection)
      * we will use the GET command 
      */
 
-    sprintf(buffer,
-	    "GET %s HTTP/1.0\r\nUser-Agent: %s%s\r\nHost: %s\r\nAccept: */*\r\n%s%s\r\n",
-	    connection->u.path, PACKAGE_NAME, VERSION,
-	    connection->u.host, referer ? referer : "",wwwauth ? wwwauth : "");
+    {
+      const char reqfmt[] =
+	"GET %s HTTP/1.0\r\n"
+	"User-Agent: %s\r\n"
+	"Host: %s\r\n"
+	"Accept: */*\r\n"
+	"%s%s\r\n";
+      int reqlen;
 
-    debug_prz("HTTP request = %s", buffer);
+      reqlen = snprintf(NULL, 0,
+			reqfmt,
+			connection->u.path, USER_AGENT,
+			connection->u.host, referer ? referer : "",
+			wwwauth ? wwwauth : "");
+
+      if (reqlen <= 0)
+	die("Unable to calculate buffer length for HTTP GET request\n");
+
+      reqlen++; /* nul */
+      req = kmalloc(reqlen);
+
+      snprintf(req, reqlen,
+	       reqfmt,
+	       connection->u.path, USER_AGENT,
+	       connection->u.host, referer ? referer : "",
+	       wwwauth ? wwwauth : "");
+    }
+
+    debug_prz("HTTP request = %s", req);
 
     err =
 	http_fetch_headers(connection->http_sock, &(connection->u),
-			   &(connection->hs), buffer);
+			   &(connection->hs), req);
 
+    kfree(req);
     if (wwwauth)
 	free(wwwauth);
 
@@ -585,8 +637,8 @@ uerr_t http_retr_fsize_known(connection_data * connection, int sock,
 			     FILE * fp)
 {
 
-    long total = 0, bytes_read = 0;
-    long temp = connection->remote_endpos - connection->remote_startpos;
+    off_t total = 0, bytes_read = 0;
+    off_t temp = connection->remote_endpos - connection->remote_startpos;
     char buffer[HTTP_BUFFER_SIZE];
 
     /* bugfix */
@@ -600,7 +652,7 @@ uerr_t http_retr_fsize_known(connection_data * connection, int sock,
 	    while (temp > 0)
 	    {
 		bytes_read =
-		    krecv(sock, buffer, HTTP_BUFFER_SIZE, 0, rt.timeout);
+		    krecv(sock, buffer, sizeof(buffer), 0, rt.timeout);
 
 		if ((bytes_read == 0 && temp > 0))
 		{
@@ -644,7 +696,7 @@ uerr_t http_retr_fsize_known(connection_data * connection, int sock,
 	    break;
 	}
 
-	bytes_read = krecv(sock, buffer, HTTP_BUFFER_SIZE, 0, rt.timeout);
+	bytes_read = krecv(sock, buffer, sizeof(buffer), 0, rt.timeout);
 
 	if ((bytes_read == 0 && temp > 0))
 	{
@@ -686,19 +738,19 @@ uerr_t http_retr_fsize_known(connection_data * connection, int sock,
     fclose(fp);
     connection->status = COMPLETED;
     message("download for this connection completed");
-    message("%s : %ld received", connection->localfile, total);
+    message("%s : %Ld received", connection->localfile, (long long) total);
     return HOK;
 }
 
 uerr_t http_retr_fsize_notknown(connection_data * connection, int sock,
 				FILE * fp)
 {
-    long bytes_read;
+    off_t bytes_read;
     char buffer[HTTP_BUFFER_SIZE];
 
     do
     {
-	bytes_read = krecv(sock, buffer, HTTP_BUFFER_SIZE, 0, rt.timeout);
+	bytes_read = krecv(sock, buffer, sizeof(buffer), 0, rt.timeout);
 	if (bytes_read > 0)
 	{
 	    if (fwrite(buffer, sizeof(char), bytes_read, fp) < bytes_read)
@@ -732,8 +784,8 @@ uerr_t http_retr_fsize_notknown(connection_data * connection, int sock,
 
     connection->status = COMPLETED;
     message("download for this connection completed");
-    message("%s : %ld received", connection->localfile,
-	    connection->remote_bytes_received);
+    message("%s : %Ld received", connection->localfile,
+	    (long long) connection->remote_bytes_received);
     return HOK;
 }
 
@@ -865,7 +917,7 @@ uerr_t fetch_http_portions(urlinfo * url_data, http_stat_t * hs,
 	}
 
       if (url_data->file_size != -1)
-	message("file size =%d", url_data->file_size);
+	message("file size =%Ld", (long long) url_data->file_size);
 
 
       /*
@@ -906,7 +958,7 @@ uerr_t fetch_http_portions(urlinfo * url_data, http_stat_t * hs,
        */
       do_logfile(url_data);
 
-      debug_prz("HTTP MAIN File size=%ld", url_data->file_size);
+      debug_prz("HTTP MAIN File size=%Ld", (long long) url_data->file_size);
 
       switch (rt.run_mode)
 	{
