@@ -20,13 +20,13 @@
 #include "ftpsearch.h"
 
 #ifdef HAVE_GTK
-int gtk_do_interface(long f_size);
+int gtk_do_interface(off_t f_size);
 #endif
 
 interface_ret curses_do_interface(connection_data * cons,
 				  int num_cons, ftp_mirror * mirrors,
 				  int num_servers);
-void curses_message(const char *args, ...);
+void curses_message(const char *msg);
 /* Displays the mesasge and gets the users input for overwriting files*/
 int curses_query_user_input(const char *args, ...);
 
