@@ -218,8 +218,8 @@ void help(void)
 	    "      -L, --license     Display software license\n"
 	    "      -v, --version     Display version number\n"
 	    "\n"
-	    "ProZilla homepage: http://prozilla.delrom.ro\n"
-	    "Please report bugs to <prozilla-dev@delrom.ro>\n");
+	    "ProZilla homepage: http://prozilla.genesys.ro\n"
+	    "Please report bugs to <prozilla@genesys.ro>\n");
 }
 
 
@@ -227,7 +227,7 @@ void help(void)
 
 void version(void)
 {
-    fprintf(stderr, "%s. Version: %s\n", PACKAGE_NAME, VERSION);
+    fprintf(stderr, "%s. Version: %s\n", PACKAGE_NAME, PACKAGE_VERSION);
 }
 
 
@@ -235,12 +235,12 @@ void version(void)
  * This function allocates the FTP connections   
  */
 
-void ftp_allocate_connections(urlinfo * u, long file_length,
+void ftp_allocate_connections(urlinfo * u, off_t file_length,
 			      char *file_io_mode)
 {
     int i;
-    long bytes_per_connection;
-    long bytes_left;
+    off_t bytes_per_connection;
+    off_t bytes_left;
 
     /*
      * Find the max path in GNU C
@@ -266,7 +266,12 @@ void ftp_allocate_connections(urlinfo * u, long file_length,
     {
 	memset(&connections[i], 0, sizeof(connection_data));
 	memcpy(&(connections[i].u), u, sizeof(urlinfo));
-	sprintf(buffer, "%s%s%d", u->file, DEFAULT_FILE_EXT, i);
+	if (snprintf(buffer, sizeof(buffer),
+		     "%s%s%d",
+		     u->file, DEFAULT_FILE_EXT, i) >= sizeof(buffer))
+	{
+	  die("Error: Filename for %s, part %d is too long\n", u->file, i);
+	}
 	connections[i].localfile = kstrdup(buffer);
 	connections[i].file_mode = kstrdup(file_io_mode);
 	connections[i].retry = TRUE;
@@ -309,12 +314,12 @@ void ftp_allocate_connections(urlinfo * u, long file_length,
  * This function allocates the HTTP connections   
  */
 
-void http_allocate_connections(urlinfo * u, long file_length,
+void http_allocate_connections(urlinfo * u, off_t file_length,
 			       char *file_io_mode)
 {
     int i;
-    long bytes_per_connection;
-    long bytes_left;
+    off_t bytes_per_connection;
+    off_t bytes_left;
 
     /*
      * Find the max path in GNU C
@@ -341,7 +346,12 @@ void http_allocate_connections(urlinfo * u, long file_length,
 	memset(&connections[i], 0, sizeof(connection_data));
 	memcpy(&(connections[i].u), u, sizeof(urlinfo));
 
-	sprintf(buffer, "%s%s%d", u->file, DEFAULT_FILE_EXT, i);
+	if (snprintf(buffer, sizeof(buffer),
+		     "%s%s%d",
+		     u->file, DEFAULT_FILE_EXT, i) >= sizeof(buffer))
+	{
+	  die("Error: Filename for %s, part %d is too long\n", u->file, i);
+	}
 	connections[i].localfile = kstrdup(buffer);
 	connections[i].file_mode = kstrdup(file_io_mode);
 	connections[i].retry = TRUE;
@@ -446,6 +456,9 @@ interface_ret do_downloads(ftp_mirror * mirrors, int num_servers)
 	pthread_join(threads[i], NULL);
     }
 
+    free(threads);
+    threads = NULL;
+
     return ret;
 }
 
@@ -502,7 +515,7 @@ void handle_threads(void)
 		 * Well the FTP server has disallowed all the connections
 		 * login attempts...so we will continue to retry this thread 
 		 */
-		message("All logins rejected!.Retrying connection");
+		message("All logins rejected! Retrying connection");
 		/*
 		 * make sure this thread has terminated 
 		 */
@@ -738,7 +751,7 @@ void handle_threads(void)
 	    break;
 
 	case LOCALFATAL:
-	    die("A local  error occured in connection %d, aborting..\n",
+	    die("A local  error occured in connection %d, aborting...\n",
 		i + 1);
 	    break;
 	default:
@@ -805,8 +818,8 @@ int query_overwrite_target(char *fname)
     {
 	ret =
 	    curses_query_user_input
-	    ("Warning: The file %s already exists!!\nWould you like"
-	     " to (O)vewrite it, (B)ackup it, (A)bort : ", p_file);
+	    ("Warning: The file %s already exists!\nWould you like"
+	     " to (A)bort, (B)ack it up, (O)verwrite it?", p_file);
     }
     while (ret != 'O' && ret != 'A' && ret != 'B');
 
@@ -831,11 +844,10 @@ int query_overwrite_target(char *fname)
 	}
 	break;
     case 'A':
-	die("Ok..Aborting...");
+	die("OK, aborting...");
 	break;
     case 'B':
-	strcpy(buffer, p_file);
-	strcat(buffer, "~");
+	snprintf(buffer, sizeof(buffer), "%s~", p_file);
 	message("Backing up %s to %s", p_file, buffer);
 	debug_prz("Backing up %s to %s", p_file, buffer);
 	if (rename(p_file, buffer) == -1)
@@ -875,7 +887,7 @@ void query_resume_old_download(urlinfo * u)
 
 	    ret =
 		curses_query_user_input
-		("Warning: Previous uncompleted download of %s detected!\n"
+		("Warning: Previous incomplete download of %s detected!\n"
 		 "Would you like to (A)bort, (R)esume it, (O)verwrite it?",
 		 u->file);
 	}
@@ -887,8 +899,8 @@ void query_resume_old_download(urlinfo * u)
 
 	    ret =
 		curses_query_user_input
-		("Warning: Previous uncompleted download of %s detected!\n"
-		 "The Server does not support resuming so would you like to (A)bort,(O)verwrite it?",
+		("Warning: Previous incomplete download of %s detected!\n"
+		 "The Server does not support resuming so would you like to (A)bort, (O)verwrite it?",
 		 u->file);
 	}
 	while (ret != 'A' && ret != 'O');
@@ -897,7 +909,7 @@ void query_resume_old_download(urlinfo * u)
     switch (ret)
     {
     case 'A':
-	die("Ok..Aborting...");
+	die("OK, aborting...");
 	break;
     case 'R':			/*
 				 *  turn the resume mode on 
@@ -919,7 +931,7 @@ void message(const char *args, ...)
     char p[MAX_MSG_SIZE];
     va_list vp;
     va_start(vp, args);
-    vsprintf(p, args, vp);
+    vsnprintf(p, sizeof(p), args, vp);
     va_end(vp);
 
     switch (rt.display_mode)
@@ -961,7 +973,7 @@ void delete_file_portions(urlinfo * u)
 	/*
 	 * Something wrong hapenned 
 	 */
-	die("Error while checking for a log file Reason: %s",
+	die("Error while checking for a log file. Reason: %s",
 	    strerror(errno));
     } else if (ret == 1)
     {
@@ -970,7 +982,7 @@ void delete_file_portions(urlinfo * u)
 	 * of connections from it
 	 */
 	if (log_read_logfile(u, &lf) != 0)
-	    die("Unable to read the log file Reason: %s", strerror(errno));
+	    die("Unable to read the log file. Reason: %s", strerror(errno));
 	else
 	{
 	    num_connections = lf.num_connections;
@@ -990,7 +1002,12 @@ void delete_file_portions(urlinfo * u)
 	 *  calculate the name of the download
 	 *  file fragment from the target file
 	 */
-	sprintf(buffer, "%s%s%d", u->file, DEFAULT_FILE_EXT, i);
+	if (snprintf(buffer, sizeof(buffer),
+		     "%s%s%d",
+		     u->file, DEFAULT_FILE_EXT, i) >= sizeof(buffer))
+	{
+	  die("Error: Filename for %s, part %d is too long\n", u->file, i);
+	}
 	ret = unlink(buffer);
 	/*
 	 *  did we encounter a error? 
@@ -1023,7 +1040,7 @@ void do_log_file_resume(urlinfo * u)
 
     assert(rt.run_mode == RESUME);
     if (log_read_logfile(u, &lf) != 0)
-	die("Unable to read the log file Reason: %s", strerror(errno));
+	die("Unable to read the log file. Reason: %s", strerror(errno));
     else
     {
 	rt.num_connections = lf.num_connections;
@@ -1051,7 +1068,7 @@ void do_logfile(urlinfo * u)
 	/*
 	 * Something wrong hapenned 
 	 */
-	die("Error while checking for a log file Reason: %s",
+       die("Error while checking for a log file. Reason: %s",
 	    strerror(errno));
     } else if (ret == 1 && rt.run_mode != RESUME && rt.force_mode == FALSE)
     {
@@ -1079,7 +1096,12 @@ void do_logfile(urlinfo * u)
 	     *  calculate the name of the download
 	     *  file fragment from the target file
 	     */
-	    sprintf(buffer, "%s%s%d", u->file, DEFAULT_FILE_EXT, i);
+	    if (snprintf(buffer, sizeof(buffer),
+			 "%s%s%d",
+			 u->file, DEFAULT_FILE_EXT, i) >= sizeof(buffer))
+	    {
+	      die("Error: Filename for %s, part %d is too long\n", u->file, i);
+	    }
 	    ret = stat(buffer, &st_buf);
 	    /*
 	     *  did we encounter a error? 
@@ -1115,7 +1137,7 @@ void do_logfile(urlinfo * u)
 		/*
 		 * Resume with 4 connections as the old version did 
 		 */
-		message("No logfile from previous session found,"
+		message("No log file from previous session found,"
 			"so I will resume with the default of 4 connections");
 		rt.num_connections = 4;
 		delay_ms(400);
@@ -1125,7 +1147,7 @@ void do_logfile(urlinfo * u)
 	 * And then lets create the log too 
 	 */
 	if (log_create_logfile(u) != 0)
-	    die("Unable to create log file Reason: %s", strerror(errno));
+	    die("Unable to create log file. Reason: %s", strerror(errno));
     }
 
 
@@ -1155,7 +1177,7 @@ int die(const char *args, ...)
     char p[MAX_MSG_SIZE];
     va_list vp;
     va_start(vp, args);
-    vsprintf(p, args, vp);
+    vsnprintf(p, sizeof(p), args, vp);
     va_end(vp);
 
     /* indicate that we are out of the display loop */
@@ -1182,7 +1204,7 @@ int die(const char *args, ...)
 	/*just dump error to stdout and quit */
 	endwin();
 	printf("\n%s\n", p);
-	exit(1);
+	exit(EXIT_FAILURE);
     }
 
     attrset(COLOR_PAIR(MSG_PAIR) | A_BOLD);
@@ -1208,7 +1230,7 @@ int die(const char *args, ...)
     while (getch() == ERR);
 
     endwin();
-    exit(1);
+    exit(EXIT_FAILURE);
 }
 
 void quit(const char *args, ...)
@@ -1216,12 +1238,12 @@ void quit(const char *args, ...)
     char p[MAX_MSG_SIZE];
     va_list vp;
     va_start(vp, args);
-    vsprintf(p, args, vp);
+    vsnprintf(p, sizeof(p), args, vp);
     va_end(vp);
 
     endwin();
     printf("\n%s\n", p);
-    exit(0);
+    exit(EXIT_SUCCESS);
 }
 
 
@@ -1271,14 +1293,20 @@ int main(int argc, char **argv)
     int same_url = FALSE;
     char *conf_file;
     char global_conf_file[] = GLOBAL_CONF_FILE;
+    int ok = 1;
+
+    memset(&hs, 0, sizeof(hs));
+    memset(&fs, 0, sizeof(fs));
+
     /*
      * set the default runtime configuration 
      */
     set_defaults(&argc, &argv);
 
-    conf_file = prozrc_file_name();
     set_preferences(global_conf_file);
-    set_preferences(conf_file);
+    conf_file = prozrc_file_name();
+    if (conf_file)
+      set_preferences(conf_file);
     debug_init();
 
     while ((c =
@@ -1289,13 +1317,13 @@ int main(int argc, char **argv)
 	{
 	case 'L':
 	    license();
-	    exit(0);
+	    exit(EXIT_SUCCESS);
 	case 'h':
 	    help();
-	    exit(0);
+	    exit(EXIT_SUCCESS);
 	case 'v':
 	    version();
-	    exit(0);
+	    exit(EXIT_SUCCESS);
 	case 'r':
 	    rt.run_mode = RESUME;
 	    break;
@@ -1310,7 +1338,7 @@ int main(int argc, char **argv)
 		 */
 		printf("Error: Invalid arguments for the -k option\n"
 		       "Please type proz --help for help\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	    }
 
 	    if (rt.num_connections == 0)
@@ -1318,7 +1346,7 @@ int main(int argc, char **argv)
 		printf("Hey! How can I download anything with 0 (Zero)"
 		       " connections!?\n"
 		       "Please type proz --help for help\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	    }
 
 	    break;
@@ -1331,7 +1359,7 @@ int main(int argc, char **argv)
 		printf
 		    ("Error: Invalid arguments for the -t or --tries option(s)\n"
 		     "Please type proz --help for help\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	    }
 	    break;
 	case 'n':
@@ -1349,7 +1377,7 @@ int main(int argc, char **argv)
 	    break;
 	case '?':
 	    help();
-	    exit(0);
+	    exit(EXIT_SUCCESS);
 	    break;
 	case '1':
 	    rt.num_connections = 1;
@@ -1365,7 +1393,7 @@ int main(int argc, char **argv)
 	    printf
 		("Error: GTK interface is not supported in "
 		 "the development version currently\n");
-	    exit(0);
+	    exit(EXIT_FAILURE);
 #endif
 	    break;
 
@@ -1387,7 +1415,7 @@ int main(int argc, char **argv)
 		printf
 		    ("Error: Invalid arguments for the --retry-delay option\n"
 		     "Please type proz --help for help\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	    }
 	    break;
 	case 131:
@@ -1400,7 +1428,7 @@ int main(int argc, char **argv)
 		printf
 		    ("Error: Invalid arguments for the --timeout option\n"
 		     "Please type proz --help for help\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	    }
 	    break;
 	case 132:
@@ -1432,7 +1460,7 @@ int main(int argc, char **argv)
 		 */
 		printf("Error: Invalid arguments for the --pt option\n"
 		       "Please type proz --help for help\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	    }
 
 	    if (rt.max_ping_wait == 0)
@@ -1441,7 +1469,7 @@ int main(int argc, char **argv)
 		    ("Hey! Does waiting for a server response for Zero(0)"
 		     " seconds make and sense to you!?\n"
 		     "Please type proz --help for help\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	    }
 
 	    break;
@@ -1454,7 +1482,7 @@ int main(int argc, char **argv)
 		 */
 		printf("Error: Invalid arguments for the --pao option\n"
 		       "Please type proz --help for help\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	    }
 
 	    if (rt.max_simul_pings == 0)
@@ -1462,7 +1490,7 @@ int main(int argc, char **argv)
 		printf("Hey you! Will pinging Zero(0) servers at once"
 		       " achive anything for me!?\n"
 		       "Please type proz --help for help\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	    }
 
 	    break;
@@ -1476,15 +1504,15 @@ int main(int argc, char **argv)
 		 */
 		printf("Error: Invalid arguments for the --pao option\n"
 		       "Please type proz --help for help\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	    }
 
 	    if (rt.ftps_mirror_req_n == 0)
 	    {
 		printf("Hey! Will requesting Zero(0) servers at once"
-		       "from tpsearch.lycos.com achive anything!?\n"
+		       "from ftpsearch.lycos.com achive anything!?\n"
 		       "Please type proz --help for help\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	    }
 
 	    break;
@@ -1498,13 +1526,13 @@ int main(int argc, char **argv)
 		printf
 		    ("Error: Invalid arguments for the --max-bps option\n"
 		     "Please type proz --help for help\n");
-		exit(0);
+		exit(EXIT_FAILURE);
 	    }
 	    break;
 
 	default:
-	    printf("Error: Invalid  option\n");
-	    exit(0);
+	    printf("Error: Invalid option\n");
+	    exit(EXIT_FAILURE);
 	}
     }
 
@@ -1512,7 +1540,7 @@ int main(int argc, char **argv)
     {
 	printf("Error: You haven't specified a URL\n"
 	       "Please type proz --help for help\n");
-	exit(0);
+	exit(EXIT_FAILURE);
     }
 
     /*
@@ -1521,11 +1549,13 @@ int main(int argc, char **argv)
     if (rt.use_netrc == TRUE)
     {
 	char *home = home_dir();
+	const char netrc[] = ".netrc";
 	if (home != NULL)
 	{
-	    char *netrc_file = malloc(strlen(home) + strlen(".netrc") + 2);
-	    sprintf(netrc_file, "%s/%s", home, ".netrc");
+	    char *netrc_file = malloc(strlen(home) + strlen(netrc) + 2);
+	    sprintf(netrc_file, "%s/%s", home, netrc);
 	    rt.netrc_list = parse_netrc(netrc_file);
+	    kfree(home);
 	}
     }
 
@@ -1548,7 +1578,7 @@ int main(int argc, char **argv)
     err = parseurl(argv[optind], &url_data, 0);
     if (err != URLOK)
     {
-	die("The URL syntatically wrong!\n");
+	die("The URL is syntactically wrong!\n");
     }
 
     if (!*url_data.file)
@@ -1568,7 +1598,7 @@ int main(int argc, char **argv)
 	     */
 	    for (i = 0; i < rt.max_redirections; ++i)
 	    {
-	char *constructed_newloc;
+	          char *constructed_newloc;
 		
 
 /*DONE : handle relative urls too */
@@ -1617,27 +1647,27 @@ int main(int argc, char **argv)
 			die("Unable to resolve %s", url_data.host);
 			break;
 		    case CONREFUSED:
-			die("%s Rejected the Connection Attempt",
+			die("%s rejected the connection attempt",
 			    url_data.host);
 			break;
 		    case CONERROR:
 			if (errno == ETIMEDOUT)
 			{
-			    die("The connection Attempt to %s timed out",
+			    die("The connection attempt to %s timed out",
 				url_data.host);
 			} else
 			    die("Error while attempting to connect to %s",
 				url_data.host);
 			break;
 		    case HAUTHREQ:
-			die("%s needs authentication to access that resource -: Privileged resource!", url_data.host);
+			die("%s needs authentication to access that resource: privileged resource!", url_data.host);
 			break;
 		    case HAUTHFAIL:
 			die("Authentication with %s failed!",
 			    url_data.host);
 			break;
 		    default:
-			die("A error occured while trying to get info from the server\n");
+			die("An error occured while trying to get info from the server\n");
 			break;
 		    }
 
@@ -1656,26 +1686,26 @@ int main(int argc, char **argv)
 		    die("Unable to resolve %s", url_data.host);
 		    break;
 		case CONREFUSED:
-		    die("%s Rejected the Connection Attempt",
+		    die("%s rejected the connection attempt",
 			url_data.host);
 		    break;
 		case CONERROR:
 		    if (errno == ETIMEDOUT)
 		    {
-			die("The connection Attempt to %s timed out",
+			die("The connection attempt to %s timed out",
 			    url_data.host);
 		    } else
 			die("Error while attempting to connect to %s",
 			    url_data.host);
 		    break;
 		case HAUTHREQ:
-		    die("%s needs authentication to access that resource -: Privileged resource", url_data.host);
+		    die("%s needs authentication to access that resource: privileged resource!", url_data.host);
 		    break;
 		case HAUTHFAIL:
 		    die("Authentication with %s failed!", url_data.host);
 		    break;
 		default:
-		    die("A error occured while trying to get info from the server\n");
+		    die("An error occured while trying to get info from the server\n");
 		    break;
 		}
 	    }
@@ -1701,19 +1731,19 @@ int main(int argc, char **argv)
 		die("Unable to resolve %s", url_data.host);
 		break;
 	    case CONREFUSED:
-		die("%s Rejected the Connection Attempt", url_data.host);
+		die("%s rejected the connection attempt", url_data.host);
 		break;
 	    case CONERROR:
 		if (errno == ETIMEDOUT)
 		{
-		    die("The connection Attempt to %s timed out",
+		    die("The connection attempt to %s timed out",
 			url_data.host);
 		} else
 		    die("Error while attempting to connect to %s",
 			url_data.host);
 		break;
 	    case FTPCONREFUSED:
-		die("%s Rejected the Connection Attempt", url_data.host);
+		die("%s rejected the connection attempt", url_data.host);
 		break;
 	    case FTPLOGREFUSED:
 		die("The FTP server has rejected the login attempt.");
@@ -1723,7 +1753,7 @@ int main(int argc, char **argv)
 		    url_data.file);
 		break;
 	    default:
-		die("A error occured while trying to get info from the server\n");
+		die("An error occured while trying to get info from the server\n");
 		break;
 	    }
 
@@ -1736,11 +1766,11 @@ int main(int argc, char **argv)
     }
     /* now we got the info lets do our ftpsearch here... */
 
-    message("File size=%ld", url_data.file_size);
+    message("File size=%Ld", (long long) url_data.file_size);
     if (url_data.file_size == -1)
     {
 	message
-	    ("Unable to get file size, Will download from original server\n");
+	    ("Unable to get file size; will download from original server\n");
 	rt.ftp_search = FALSE;
     }
 
@@ -1753,7 +1783,7 @@ int main(int argc, char **argv)
 	if (err != MIRINFOK)
 	{
 	    message
-		("Unable to get mirror info, Will download from original server\n");
+		("Unable to get mirror info; will download from original server\n");
 	    same_url = TRUE;
 	    delay_ms(2000);
 	} else
@@ -1761,7 +1791,7 @@ int main(int argc, char **argv)
 	    if (num_servers == 0)
 	    {
 		message
-		    ("No mirrors were found, Will download from original server\n");
+		    ("No mirrors were found; will download from original server\n");
 		same_url = TRUE;
 		delay_ms(2000);
 	    } else
@@ -1891,7 +1921,7 @@ int main(int argc, char **argv)
     }
 
 
-    message("File Succesfully Retreived, Now joining the sections");
+    message("File Successfully Retreived, Now joining the sections");
     join_downloads(url_data.file, connections, rt.num_connections);
 
     message("Deleting the unwanted files");
@@ -1901,14 +1931,16 @@ int main(int argc, char **argv)
 
     if (log_delete_logfile(&url_data) != 0)
     {
-	die("Error while deleting the logfile: %s", strerror(errno));
-	return 1;
-    } else
-	quit("All Done: Download Succesfull!");
+	die("Error while deleting the log file: %s", strerror(errno));
+	ok = 0;
+    }
+
+    if (ok)
+      quit("All Done: Download Successful!");
 
 
     /*
      *  just to keep gcc quiet 
      */
-    return 1;
+    return ok ? EXIT_SUCCESS : EXIT_FAILURE;
 }
