@@ -22,9 +22,9 @@
 #include "url.h"
 #include "ftpsearch.h"
 
-void http_allocate_connections(urlinfo * u, long file_length,
+void http_allocate_connections(urlinfo * u, off_t file_length,
 			       char *file_io_mode);
-void ftp_allocate_connections(urlinfo * u, long file_length,
+void ftp_allocate_connections(urlinfo * u, off_t file_length,
 			      char *file_io_mode);
 void do_logfile(urlinfo * u);
 interface_ret do_downloads(ftp_mirror * mirrors, int num_servers);
