@@ -130,7 +130,7 @@ enum {
 
 #define ARRAY_SIZE(array) (sizeof (array) / sizeof (*(array)))
 
-const static unsigned char urlchr_table[256] = {
+static const unsigned char urlchr_table[256] = {
   U, U, U, U, U, U, U, U,	/* NUL SOH STX ETX  EOT ENQ ACK BEL */
   U, U, U, U, U, U, U, U,	/* BS  HT  LF  VT   FF  CR  SO  SI  */
   U, U, U, U, U, U, U, U,	/* DLE DC1 DC2 DC3  DC4 NAK SYN ETB */
@@ -238,7 +238,7 @@ char *encode_string_maybe(const char *s)
     return (char *) s;
 
   newlen = (p1 - s) + addition;
-  newstr = (char *) kmalloc(newlen + 1);
+  newstr = kmalloc(newlen + 1);
 
   p1 = s;
   p2 = newstr;
@@ -561,7 +561,7 @@ static uerr_t parse_uname(const char *url, char **user, char **passwd)
   {
     if (*p == ':' && !*user)
     {
-      *user = (char *) kmalloc(p - url + 1);
+      *user = kmalloc(p - url + 1);
       memcpy(*user, url, p - url);
       (*user)[p - url] = '\0';
       col = p + 1;
@@ -571,7 +571,7 @@ static uerr_t parse_uname(const char *url, char **user, char **passwd)
   }
   /* Decide whether you have only the username or both.  */
   where = *user ? passwd : user;
-  *where = (char *) kmalloc(q - col + 1);
+  *where = kmalloc(q - col + 1);
   memcpy(*where, col, q - col);
   (*where)[q - col] = '\0';
   return URLOK;
@@ -615,7 +615,7 @@ char *str_url(const urlinfo * u, int hide)
   }
   if (u->proto == URLFTP && *dir == '/')
   {
-    char *tmp = (char *) kmalloc(strlen(dir) + 3);
+    char *tmp = kmalloc(strlen(dir) + 3);
     /*sprintf (tmp, "%%2F%s", dir + 1); */
     tmp[0] = '%';
     tmp[1] = '2';
@@ -631,7 +631,7 @@ char *str_url(const urlinfo * u, int hide)
   lh = strlen(host);
   ld = strlen(dir);
   lf = strlen(file);
-  res = (char *) kmalloc(ln + lu + lp + lh + ld + lf + 20);	/* safe sex */
+  res = kmalloc(ln + lu + lp + lh + ld + lf + 20);	/* safe sex */
   /* sprintf (res, "%s%s%s%s%s%s:%d/%s%s%s", proto_name,
      (user ? user : ""), (passwd ? ":" : ""),
      (passwd ? passwd : ""), (user ? "@" : ""),
@@ -773,7 +773,7 @@ uerr_t parseurl(const char *url, urlinfo * u, int strict)
   if (type == URLHTTP)
     while (url[i] && url[i] == '/')
       ++i;
-  u->path = (char *) kmalloc(strlen(url + i) + 8);
+  u->path = kmalloc(strlen(url + i) + 8);
   strcpy(u->path, url + i);
   if (type == URLFTP)
   {
@@ -903,14 +903,6 @@ urlpos *add_url(urlpos * l, const char *url, const char *file)
 }
 
 
-/*This will copy a url structure to another */
-void url_cpy(urlinfo * src, urlinfo * dest)
-{
-
-
-}
-
-
 /* Find the last occurrence of character C in the range [b, e), or
    NULL, if none are present.  This is almost completely equivalent to
    { *e = '\0'; return strrchr(b); }, except that it doesn't change
@@ -991,7 +983,7 @@ char *uri_merge_1(const char *base, const char *link, int linklength,
       }
 
       span = start_insert - base;
-      constr = (char *) kmalloc(span + linklength + 1);
+      constr = kmalloc(span + linklength + 1);
       if (span)
 	memcpy(constr, base, span);
       if (need_explicit_slash)
@@ -1048,7 +1040,7 @@ char *uri_merge_1(const char *base, const char *link, int linklength,
 	start_insert = slash;
 
       span = start_insert - base;
-      constr = (char *) kmalloc(span + linklength + 1);
+      constr = kmalloc(span + linklength + 1);
       if (span)
 	memcpy(constr, base, span);
       if (linklength)
